'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _cliTable = require('cli-table2');

var _cliTable2 = _interopRequireDefault(_cliTable);

var _open = require('open');

var _open2 = _interopRequireDefault(_open);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _api = require('./api');

var _user = require('./user');

var _app = require('./app');

var _package = require('./package');

var _versions = require('./versions');

var _bundle = require('./bundle');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _ = require('lodash');

var program = require('./_commander');
var NOOP = function NOOP() {};

var PACKAGE_JSON_PATH = function PACKAGE_JSON_PATH() {
  return _path2.default.resolve(process.cwd(), 'package.json');
};

// 获取并检查 react-native 项目依赖
var rn_version = void 0;
try {
  var package_info = require(PACKAGE_JSON_PATH());
  rn_version = package_info.dependencies['react-native'];
  if (!rn_version) {
    throw new Error('not found react-native dependencies');
  }
} catch (e) {
  console.log(_colors2.default.green('rnkit-code-push: ') + _colors2.default.red('没找到React-Native配置信息, 请切换至RN项目目录运行此命令!'));
  console.error(_colors2.default.red(e));
  process.exit(1);
}

var cmd = function cmd() {
  program.version(require('../package.json').version, '-v, --version');
  process.argv = _.map(process.argv, function (arg) {
    return arg === '-V' ? '-v' : arg;
  });

  var lastArg = _.last(process.argv);
  if (lastArg === 'app' || lastArg === 'bundle' || lastArg === 'publish') {
    process.argv.push('-h');
  }

  program.command('login').description('Authenticate with the rnkit-code-push server in order to begin managing your apps').action(function () {
    _inquirer2.default.prompt([{
      type: 'input',
      message: 'Enter your email',
      name: 'email',
      validate: function validate(value) {
        var pass = _validator2.default.isEmail(value);
        if (pass) {
          return true;
        }
        return 'Please enter a valid email';
      }
    }, {
      type: 'password',
      message: 'Enter your password',
      name: 'password',
      validate: function validate(value) {
        var pass = value.length >= 6;
        if (pass) {
          return true;
        }
        return 'Please enter a valid password (6-20)';
      }
    }]).then(function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(answers) {
        var email, password;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                email = answers.email, password = answers.password;
                _context.prev = 1;
                _context.next = 4;
                return (0, _user.login)(email, password);

              case 4:
                _context.next = 9;
                break;

              case 6:
                _context.prev = 6;
                _context.t0 = _context['catch'](1);

                console.log(_context.t0.message);

              case 9:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined, [[1, 6]]);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }());
  });

  program.command('register').description('Register a new rnkit-code-push account').action((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            (0, _open2.default)('https://update.rnkit.io');
            process.exit(-1);

          case 2:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  program.command('logout').description('Log out of the current session').action((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.prev = 0;
            _context3.next = 3;
            return (0, _user.logout)();

          case 3:
            _context3.next = 8;
            break;

          case 5:
            _context3.prev = 5;
            _context3.t0 = _context3['catch'](0);

            console.log(_context3.t0);

          case 8:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined, [[0, 5]]);
  })));

  program.command('whoami').description('Display the account info for the current login session').action((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
    return _regenerator2.default.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.prev = 0;
            _context4.next = 3;
            return (0, _user.me)();

          case 3:
            _context4.next = 8;
            break;

          case 5:
            _context4.prev = 5;
            _context4.t0 = _context4['catch'](0);

            console.log(_context4.t0);

          case 8:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined, [[0, 5]]);
  })));

  program.command('app').option('list', 'Lists the apps associated with your account').option('ls', 'Lists the apps associated with your account').description('View and manage your rnkit-code-push apps').action(function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(arg) {
      var data, table, k, platform;
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              if (!(arg === 'list' || arg === 'ls')) {
                _context5.next = 14;
                break;
              }

              _context5.prev = 1;
              _context5.next = 4;
              return (0, _app.listApp)();

            case 4:
              data = _context5.sent;
              table = new _cliTable2.default({
                head: ['AppName', 'Platform', 'AppIdentifier', 'AppKey', 'CreatedAt']
              });

              for (k in data) {
                platform = data[k].platform === 1 ? 'ios' : 'android';

                table.push(['' + data[k].name, '' + platform, '' + data[k].app_identifier, '' + data[k].key, '' + data[k].created_at]);
              }
              console.log('\nTotal ' + data.length + ' apps');
              console.log(_colors2.default.green(table.toString()));
              _context5.next = 14;
              break;

            case 11:
              _context5.prev = 11;
              _context5.t0 = _context5['catch'](1);

              console.log(_context5.t0);

            case 14:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, undefined, [[1, 11]]);
    }));

    return function (_x2) {
      return _ref5.apply(this, arguments);
    };
  }()).parse(process.argv);

  program.command('bindApp <platform>').description('The binding is applied to the current project with platform').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(platform, options) {
      var result, data, list, k;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.prev = 0;
              result = (0, _app.checkPlatform)(platform);
              _context7.next = 4;
              return (0, _app.listApp)(result === 'ios' ? 1 : 2);

            case 4:
              data = _context7.sent;

              if (!data.length) {
                console.log('You Don\'t An App For ' + platform + ' platform');
                process.exit(-1);
              }
              list = [];

              for (k in data) {
                list.push({
                  name: data[k].name
                });
              }
              _inquirer2.default.prompt([{
                type: 'rawlist',
                name: 'app',
                message: 'Please Choose App :',
                choices: list
              }]).then(function () {
                var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(answers) {
                  var obj;
                  return _regenerator2.default.wrap(function _callee6$(_context6) {
                    while (1) {
                      switch (_context6.prev = _context6.next) {
                        case 0:
                          _context6.prev = 0;
                          obj = data.find(function (v) {
                            return v.name === answers.app;
                          });
                          _context6.next = 4;
                          return (0, _app.selectApp)(obj.name, obj.key, platform, options.app_key_path);

                        case 4:
                          console.log('You choose: ' + obj.name);
                          _context6.next = 10;
                          break;

                        case 7:
                          _context6.prev = 7;
                          _context6.t0 = _context6['catch'](0);

                          console.log(_colors2.default.red(_context6.t0.message));

                        case 10:
                        case 'end':
                          return _context6.stop();
                      }
                    }
                  }, _callee6, undefined, [[0, 7]]);
                }));

                return function (_x5) {
                  return _ref7.apply(this, arguments);
                };
              }());
              _context7.next = 14;
              break;

            case 11:
              _context7.prev = 11;
              _context7.t0 = _context7['catch'](0);

              console.log(_colors2.default.red(_context7.t0.message));

            case 14:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, undefined, [[0, 11]]);
    }));

    return function (_x3, _x4) {
      return _ref6.apply(this, arguments);
    };
  }()).parse(process.argv);

  program.command('uploadIpa <ipaFilePath>').description('upload ios ipa file to rnkit code push server').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(ipaFilePath, options) {
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              _context8.prev = 0;
              _context8.next = 3;
              return (0, _package.uploadIpa)(ipaFilePath, options.app_key_path);

            case 3:
              _context8.next = 8;
              break;

            case 5:
              _context8.prev = 5;
              _context8.t0 = _context8['catch'](0);

              console.log(_colors2.default.red(_context8.t0.message));

            case 8:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined, [[0, 5]]);
    }));

    return function (_x6, _x7) {
      return _ref8.apply(this, arguments);
    };
  }());

  program.command('uploadApk <apkFilePath>').description('upload android apk file to rnkit code push server').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(apkFilePath, options) {
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.prev = 0;
              _context9.next = 3;
              return (0, _package.uploadApk)(apkFilePath, options.app_key_path);

            case 3:
              _context9.next = 8;
              break;

            case 5:
              _context9.prev = 5;
              _context9.t0 = _context9['catch'](0);

              console.log(_colors2.default.red(_context9.t0.message));

            case 8:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, undefined, [[0, 5]]);
    }));

    return function (_x8, _x9) {
      return _ref9.apply(this, arguments);
    };
  }());

  program.command('packages <platform>').description('View the packages for a running app').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(platform, options) {
      var app_info, data, table, k, version, _platform;

      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.prev = 0;
              _context10.next = 3;
              return (0, _app.checkPlatform)(platform);

            case 3:
              _context10.next = 5;
              return (0, _app.getSelectedApp)(platform, options.app_key_path);

            case 5:
              app_info = _context10.sent;
              _context10.next = 8;
              return (0, _package.listPackage)(app_info.appKey);

            case 8:
              data = _context10.sent;
              table = new _cliTable2.default({
                head: ['name', 'app_version', 'platform', 'key', 'bind_version', 'created_at']
              });

              for (k in data) {
                version = data[k].version.map(function (obj, index) {
                  return obj.name;
                });
                _platform = data[k].platform === 1 ? 'ios' : 'android';

                table.push(['' + data[k].name, '' + data[k].app_version, '' + _platform, '' + data[k].key, '' + version, '' + data[k].created_at]);
              }
              console.log('\nTotal ' + data.length + ' packages.');
              console.log(_colors2.default.green(table.toString()));
              _context10.next = 18;
              break;

            case 15:
              _context10.prev = 15;
              _context10.t0 = _context10['catch'](0);

              console.log(_colors2.default.red(_context10.t0.message));

            case 18:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, undefined, [[0, 15]]);
    }));

    return function (_x10, _x11) {
      return _ref10.apply(this, arguments);
    };
  }());

  program.command('versions <platform>').description('View the versions for a running app').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(platform, options) {
      var app_info;
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              _context11.prev = 0;
              _context11.next = 3;
              return (0, _app.checkPlatform)(platform);

            case 3:
              _context11.next = 5;
              return (0, _app.getSelectedApp)(platform, options.app_key_path);

            case 5:
              app_info = _context11.sent;
              _context11.next = 8;
              return (0, _versions.listVersions)(app_info.appKey);

            case 8:
              _context11.next = 13;
              break;

            case 10:
              _context11.prev = 10;
              _context11.t0 = _context11['catch'](0);

              console.log(_colors2.default.red(_context11.t0.message));

            case 13:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, undefined, [[0, 10]]);
    }));

    return function (_x12, _x13) {
      return _ref11.apply(this, arguments);
    };
  }());

  program.command('publish').description('publish version to rnkit-code-push').option('-p --platform <platform>', 'platform ios|android').option('-f --ppkFile <ppkFile>', 'ppkFile Path').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').on('--help', function () {
    console.log(_colors2.default.bold('  Examples:'));
    console.log('');
    console.log(_colors2.default.magenta('  $rnkit-code-push --platform ios --ppkFile /data/my-app/build/output/android.xxxxx.ppk'));
    console.log('');
  }).action(function () {
    var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(args) {
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              _context12.prev = 0;
              _context12.next = 3;
              return (0, _versions.publish)(args);

            case 3:
              _context12.next = 8;
              break;

            case 5:
              _context12.prev = 5;
              _context12.t0 = _context12['catch'](0);

              console.log(_colors2.default.red(_context12.t0.message));

            case 8:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, undefined, [[0, 5]]);
    }));

    return function (_x14) {
      return _ref12.apply(this, arguments);
    };
  }());

  program.command('deployment').description('deployment version to packages').option('-p --platform <platform>', 'platform ios|android').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(args) {
      return _regenerator2.default.wrap(function _callee13$(_context13) {
        while (1) {
          switch (_context13.prev = _context13.next) {
            case 0:
              _context13.prev = 0;
              _context13.next = 3;
              return (0, _package.deployment)(args.platform, args.app_key_path);

            case 3:
              _context13.next = 8;
              break;

            case 5:
              _context13.prev = 5;
              _context13.t0 = _context13['catch'](0);

              console.log(_colors2.default.red(_context13.t0.message));

            case 8:
            case 'end':
              return _context13.stop();
          }
        }
      }, _callee13, undefined, [[0, 5]]);
    }));

    return function (_x15) {
      return _ref13.apply(this, arguments);
    };
  }());

  program.command('bundle').description('builds the javascript bundle for offline use').option('--dev <dev>', 'If false, warnings are disabled and the bundle is minified').option('--platform <platform>', 'Either "ios" or "android"').option('--entryFile <entryFile>', 'Path to the root JS file, either absolute or relative to JS root').option('--output <output>', 'File name where to store the resulting bundle, ex. /tmp/groups.bundle').option('--intermediaDir <intermediaDir>', 'tmp file out dir').option('--verbose <verbose>', 'Enables logging').option('-k, --app_key_path <app_key_path>', 'the json file path of app_key').action(function () {
    var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14(args) {
      return _regenerator2.default.wrap(function _callee14$(_context14) {
        while (1) {
          switch (_context14.prev = _context14.next) {
            case 0:
              _context14.prev = 0;
              _context14.next = 3;
              return (0, _bundle.bundleApp)(args);

            case 3:
              _context14.next = 8;
              break;

            case 5:
              _context14.prev = 5;
              _context14.t0 = _context14['catch'](0);

              console.log(_colors2.default.red(_context14.t0.message));

            case 8:
            case 'end':
              return _context14.stop();
          }
        }
      }, _callee14, undefined, [[0, 5]]);
    }));

    return function (_x16) {
      return _ref14.apply(this, arguments);
    };
  }());

  program.command('web').description('open rnkit-code-push website').action(function () {
    (0, _open2.default)('https://update.rnkit.io');
    process.exit(-1);
  });

  program.unknownOption = NOOP;
  program.parse(process.argv);
  var NO_COMMAND_SPECIFIED = program.args.length === 0;
  if (NO_COMMAND_SPECIFIED) {
    program.usageMinusWildcard();
  }
};

(0, _api.loadToken)().then(function () {
  cmd();
}).catch(function (err) {
  if (err.status === 401) {
    console.log('Not loggined.\nRun `rnkit-code-push login` at your project directory to login.');
    return;
  }
  console.error(err.message);
  process.exit(-1);
});