'use strict';

exports.__esModule = true;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.question = question;
exports.translateOptions = translateOptions;
exports.getRNVersion = getRNVersion;
exports.getRNPackage = getRNPackage;
exports.getApkVersion = getApkVersion;
exports.getIPAVersion = getIPAVersion;

var _path = require('path');

var path = _interopRequireWildcard(_path);

var _fs = require('fs');

var fs = _interopRequireWildcard(_fs);

var _nodeApkParser = require('node-apk-parser');

var _nodeApkParser2 = _interopRequireDefault(_nodeApkParser);

var _ipaMetadata = require('ipa-metadata');

var _ipaMetadata2 = _interopRequireDefault(_ipaMetadata);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var read = require('read');

function question(query, password) {
  return new _promise2.default(function (resolve, reject) {
    return read({
      prompt: query,
      silent: password,
      replace: password ? '*' : undefined
    }, function (err, result) {
      return err ? reject(err) : resolve(result);
    });
  });
}

function translateOptions(options) {
  var ret = {};
  for (var key in options) {
    var v = options[key];
    if (typeof v === 'string') {
      ret[key] = v.replace(/\$\{(\w+)\}/g, function (v, n) {
        return options[n] || process.env[n] || v;
      });
    } else {
      ret[key] = v;
    }
  }
  return ret;
}

// return { version: '0.27.2', major: 0, minor: 27 }
function getRNVersion() {
  var version = JSON.parse(fs.readFileSync(path.resolve('node_modules/react-native/package.json'))).version;

  // We only care about major and minor version.
  var match = /^(\d+)\.(\d+)\./.exec(version);
  return {
    version: version,
    major: match[1] | 0,
    minor: match[2] | 0
  };
}

function getRNPackage() {
  var pkg = JSON.parse(fs.readFileSync(path.resolve('package.json')));
  return pkg;
}

function getApkVersion(fn) {
  var reader = _nodeApkParser2.default.readFile(fn);
  var manifest = reader.readManifestSync();
  return _promise2.default.resolve(manifest.versionName);
}

function getIPAVersion(fn) {
  return new _promise2.default(function (resolve, reject) {
    (0, _ipaMetadata2.default)(fn, function (err, data) {
      err ? reject(err) : resolve(data.metadata.CFBundleShortVersionString);
    });
  });
}